CodePoint

Enum CodePoint 

pub enum CodePoint {
    Unicode(char),
    UnpairedSurrogate(u16),
}
Represents a Unicode codepoint within a crate::JsString, which could be a valid ‘Unicode scalar value’, or an unpaired surrogate.

Variants



Unicode(char)

A valid Unicode scalar value.



UnpairedSurrogate(u16)

An unpaired surrogate.

Implementations

Source

impl CodePoint

pub const fn code_unit_count(self) -> usize

Get the number of UTF-16 code units needed to encode this code point.

pub fn as_u32(self) -> u32

Convert the code point to its u32 representation.

pub const fn as_char(self) -> Option<char>

If the code point represents a valid ‘Unicode scalar value’, returns its char representation, otherwise returns None on unpaired surrogates.

pub fn encode_utf16(self, dst: &mut [u16]) -> &mut [u16]

Encodes this code point as UTF-16 into the provided u16 buffer, and then returns the subslice of the buffer that contains the encoded character.

Panics

Panics if the buffer is not large enough. A buffer of length 2 is large enough to encode any code point.