JsString

Struct JsString 

pub struct JsString { /* private fields */ }
A Latin1 or UTF-16–encoded, reference counted, immutable string.

This is pretty similar to a Rc<[u16]>, but without the length metadata associated with the Rc fat pointer. Instead, the length of every string is stored on the heap, along with its reference counter and its data.

The string can be latin1 (stored as a byte for space efficiency) or U16 encoding.

We define some commonly used string constants in an interner. For these strings, we don’t allocate memory on the heap to reduce the overhead of memory allocation and reference counting.

Internal representation

The ptr field always points to a structure whose first field is a JsStringVTable. This enables uniform vtable dispatch for all string operations without branching.

Because we ensure this invariant at every construction, we can directly point to this type to allow for better optimization (and simpler code).

Implementations

Source

impl JsString

pub fn iter(&self) -> Iter<'_>

Create an iterator over the JsString.

pub fn windows(&self, size: usize) -> Windows<'_>

Create an iterator over overlapping subslices of length size.

pub fn to_std_string_escaped(&self) -> String

Decodes a JsString into a String, replacing invalid data with its escaped representation in 4 digit hexadecimal.

pub fn to_std_string_lossy(&self) -> String

Decodes a JsString into a String, replacing invalid data with the replacement character U+FFFD.

pub fn to_std_string(&self) -> Result<String, FromUtf16Error>

Decodes a JsString into a String, returning an error if the string contains unpaired surrogates.

Errors

FromUtf16Error if it contains any invalid data.

pub fn to_std_string_with_surrogates( &self, ) -> impl Iterator<Item = Result<String, u16>> + use<'_>

Decodes a JsString into an iterator of Result<String, u16>, returning surrogates as errors.

pub fn map_valid_segments<F>(&self, f: F) -> Self
where F: FnMut(String) -> String,

Maps the valid segments of an UTF16 string and leaves the unpaired surrogates unchanged.

pub fn code_points(&self) -> CodePointsIter<'_>

Gets an iterator of all the Unicode codepoints of a JsString.

pub fn variant(&self) -> JsStrVariant<'_>

Get the variant of this string.

pub fn index_of( &self, search_value: JsStr<'_>, from_index: usize, ) -> Option<usize>

Abstract operation StringIndexOf ( string, searchValue, fromIndex )

Note: Instead of returning an isize with -1 as the “not found” value, we make use of the type system and return Option<usize> with None as the “not found” value.

More information:

pub fn code_point_at(&self, position: usize) -> CodePoint

Abstract operation CodePointAt( string, position ).

The abstract operation CodePointAt takes arguments string (a String) and position (a non-negative integer) and returns a Record with fields [[CodePoint]] (a code point), [[CodeUnitCount]] (a positive integer), and [[IsUnpairedSurrogate]] (a Boolean). It interprets string as a sequence of UTF-16 encoded code points, as described in 6.1.4, and reads from it a single code point starting with the code unit at index position.

More information:

Panics

If position is smaller than size of string.

pub fn to_number(&self) -> f64

Abstract operation StringToNumber ( str )

More information:

pub fn len(&self) -> usize

Get the length of the JsString.

pub fn is_empty(&self) -> bool

Return true if the JsString is empty.

pub fn to_vec(&self) -> Vec<u16>

Convert the JsString into a Vec<U16>.

pub fn contains(&self, element: u8) -> bool

Check if the JsString contains a byte.

pub fn trim(&self) -> JsString

Trim whitespace from the start and end of the JsString.

pub fn trim_start(&self) -> JsString

Trim whitespace from the start of the JsString.

pub fn trim_end(&self) -> JsString

Trim whitespace from the end of the JsString.

pub fn starts_with(&self, needle: JsStr<'_>) -> bool

Returns true if needle is a prefix of the JsStr.

pub fn ends_with(&self, needle: JsStr<'_>) -> bool

Returns true if needle is a suffix of the JsStr.

pub fn code_unit_at(&self, index: usize) -> Option<u16>

Get the u16 code unit at index. This does not parse any characters if there are pairs, it is simply the index of the u16 elements.

pub fn get<I>(&self, index: I) -> Option<JsString>

Get the element at the given index, or None if the index is out of range.

pub fn get_expect<I>(&self, index: I) -> JsString

Get the element at the given index, or panic.

Panics

If the index returns None, this will panic.

pub fn display_escaped(&self) -> JsStrDisplayEscaped<'_>

Gets a displayable escaped string. This may be faster and has fewer allocations than format!("{}", str.to_string_escaped()) when displaying.

pub fn display_lossy(&self) -> JsStrDisplayLossy<'_>

Gets a displayable lossy string. This may be faster and has fewer allocations than format!("{}", str.to_string_lossy()) when displaying.

pub fn debug_info(&self) -> JsStringDebugInfo<'_>

Get a debug displayable info and metadata for this string.

pub fn into_raw(self) -> NonNull<RawJsString>

Consumes the JsString, returning the internal pointer.

To avoid a memory leak the pointer must be converted back to a JsString using JsString::from_raw.

pub const unsafe fn from_raw(ptr: NonNull<RawJsString>) -> Self

Constructs a JsString from the internal pointer.

The raw pointer must have been previously returned by a call to JsString::into_raw.

Safety

This function is unsafe because improper use may lead to memory unsafety, even if the returned JsString is never accessed.

Source

impl JsString

Dealing with inner types.

pub fn is_static(&self) -> bool

Check if this is a static string.

pub const fn from_static(str: &'static StaticString) -> Self

Create a JsString from a StaticString instance. This is assumed that the static string referenced is available for the duration of the JsString instance returned.

pub unsafe fn slice_unchecked(data: &JsString, start: usize, end: usize) -> Self

Create a JsString from an existing JsString and start, end range. end is 1 past the last character (or == data.len() for the last character).

Safety

It is the responsibility of the caller to ensure:

  • start <= end. If start == end, the string is empty.
  • end <= data.len().
pub fn slice(&self, p1: usize, p2: usize) -> JsString

Create a JsString from an existing JsString and start, end range. Returns None if the start/end is invalid.

Source

impl JsString

pub fn as_str(&self) -> JsStr<'_>

Obtains the underlying &[u16] slice of a JsString

pub fn concat(x: JsStr<'_>, y: JsStr<'_>) -> Self

Creates a new JsString from the concatenation of x and y.

pub fn concat_array(strings: &[JsStr<'_>]) -> Self

Creates a new JsString from the concatenation of every element of strings.

pub fn refcount(&self) -> Option<usize>

Gets the number of JsStrings which point to this allocation.